@@ -4,10 +4,11 @@
 # for our purposes. Thanks guys!
 
 echo "This will do all the autotools stuff so that you can build"
-echo "tilda successfully."
+echo "tilda successfully. It will also call the ./configure script"
+echo "and pass on any options which you passed to this script"
+echo "See ./configure --help to know which options are available"
 echo
 echo "When it is finished, take the usual steps to install:"
-echo "./configure"
 echo "make"
 echo "make install"
 echo
@@ -29,9 +30,18 @@ echo
 echo "Generating configuration files for tilda, please wait..."
 echo;
 
-aclocal $ACLOCAL_FLAGS || exit;
-autoheader || exit;
-automake --add-missing --copy;
-autoconf || exit;
-automake || exit;
+# Autoconf will call run autopoint, aclocal, autoconf, autoheader and automake
+# to setup and configure the build environment
+autoreconf --verbose --install --symlink --force || {
+    echo;
+    echo "autoreconf has encountered an error."
+    echo;
+    exit $?
+}
+# Afterwards we invoke the configure skript, "$@" will contain the arguments that
+# were passed to this skript.
 
+echo
+echo "Running configure now"
+echo
+./configure "$@"
