@@ -23,7 +23,8 @@
 #include <key_grabber.h>
 #include <tilda.h>
 #include <xerror.h>
-#include <translation.h>
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <configsys.h>
 #include <tomboykeybinder.h>
 
@@ -31,7 +32,7 @@
 #include <X11/keysym.h>
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
-#include <gtk/gtkmain.h>
+#include <gtk/gtk.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -98,6 +99,7 @@ void generate_animation_positions (struct tilda_window_ *tw)
             posIV[2][i] = last_pos_x;
             posIV[1][i] = (gint)(posCFV[i]*last_height);
             posIV[0][i] = last_pos_y;
+            break;
         }
     }
 }
@@ -123,9 +125,9 @@ void tilda_window_set_active (tilda_window *tw)
     DEBUG_FUNCTION ("tilda_window_set_active");
     DEBUG_ASSERT (tw != NULL);
 
-    Display *x11_display = GDK_WINDOW_XDISPLAY( tw->window->window );
-    Window *x11_window = GDK_WINDOW_XWINDOW( tw->window->window );
-    Window *x11_root_window = GDK_WINDOW_XWINDOW ( gtk_widget_get_root_window (tw->window) );
+    Display *x11_display = GDK_WINDOW_XDISPLAY (gtk_widget_get_window (tw->window) );
+    Window x11_window = GDK_WINDOW_XID (gtk_widget_get_window (tw->window) );
+    Window x11_root_window = GDK_WINDOW_XID ( gtk_widget_get_root_window (tw->window) );
     GdkScreen *screen = gtk_widget_get_screen (tw->window);
 
     XEvent event;
@@ -190,7 +192,7 @@ void pull (struct tilda_window_ *tw, enum pull_state state)
          * Overriding the user time here seems to work a lot better than calling
          * gtk_window_present_with_time() here, or at the end of the function. I have
          * no idea why, they should do the same thing. */
-        gdk_x11_window_set_user_time (GTK_WIDGET(tw->window)->window,
+        gdk_x11_window_set_user_time (gtk_widget_get_window (tw->window),
                                       tomboy_keybinder_get_current_event_time());
         gtk_window_move (GTK_WINDOW(tw->window), config_getint ("x_pos"), config_getint ("y_pos"));
         gtk_widget_show (GTK_WIDGET(tw->window));
@@ -257,15 +259,15 @@ static void onKeybindingPull (const char *keystring, gpointer user_data)
 gboolean tilda_keygrabber_bind (const gchar *keystr, tilda_window *tw)
 {
     /* Empty strings are no good */
-    if (strcmp ("", keystr) == 0)
+    if (keystr == NULL || strcmp ("", keystr) == 0)
         return FALSE;
 
-    return tomboy_keybinder_bind (keystr, onKeybindingPull, tw);
+    return tomboy_keybinder_bind (keystr, (TomboyBindkeyHandler)onKeybindingPull, tw);
 }
 
 void tilda_keygrabber_unbind (const gchar *keystr)
 {
-    tomboy_keybinder_unbind (keystr, onKeybindingPull);
+    tomboy_keybinder_unbind (keystr, (TomboyBindkeyHandler)onKeybindingPull);
 }
 
 
