@@ -24,7 +24,6 @@
 #include <key_grabber.h> /* for pull */
 #include <wizard.h>
 #include <xerror.h>
-#include <translation.h>
 
 #include <sys/ioctl.h>
 #include <sys/stat.h>
@@ -37,6 +36,8 @@
 #include <gtk/gtk.h>
 #include <glib-object.h>
 #include <glib.h>
+#include <glib/gi18n.h>
+#include <locale.h>
 #include <stdio.h>
 #include <signal.h>
 #include <pwd.h>
@@ -50,26 +51,25 @@
 #include <glib/gstdio.h>
 
 
-static gchar *create_lock_file (gchar *home_directory, struct lock_info lock)
+static gchar *create_lock_file (struct lock_info lock)
 {
     DEBUG_FUNCTION ("create_lock_file");
-    DEBUG_ASSERT (home_directory != NULL);
     DEBUG_ASSERT (lock.instance >= 0);
     DEBUG_ASSERT (lock.pid >= 1);
 
     gint ret;
     gchar *lock_file_full;
-    gchar *lock_directory = g_build_filename (home_directory, ".tilda", "locks", NULL);
+    gchar *lock_dir = g_build_filename (g_get_user_cache_dir (), "tilda", "locks", NULL);
     gchar *lock_file = g_strdup_printf ("lock_%d_%d", lock.pid, lock.instance);
 
-    /* Make the ~/.tilda/locks directory */
-    ret = g_mkdir_with_parents (lock_directory,  S_IRUSR | S_IWUSR | S_IXUSR);
+    /* Make the ~/.cache/tilda/locks directory */
+    ret = g_mkdir_with_parents (lock_dir,  S_IRUSR | S_IWUSR | S_IXUSR);
 
     if (ret == -1)
         goto mkdir_fail;
 
     /* Create the full path to the lock file */
-    lock_file_full = g_build_filename (lock_directory, lock_file, NULL);
+    lock_file_full = g_build_filename (lock_dir, lock_file, NULL);
 
     /* Create the lock file */
     ret = g_creat (lock_file_full, S_IRUSR | S_IWUSR | S_IXUSR);
@@ -78,7 +78,7 @@ static gchar *create_lock_file (gchar *home_directory, struct lock_info lock)
         goto creat_fail;
 
     g_free (lock_file);
-    g_free (lock_directory);
+    g_free (lock_dir);
 
     return lock_file_full;
 
@@ -87,7 +87,7 @@ creat_fail:
     g_free (lock_file_full);
 mkdir_fail:
     g_free (lock_file);
-    g_free (lock_directory);
+    g_free (lock_dir);
 
     return NULL;
 }
@@ -194,15 +194,12 @@ nomatch:
 /**
  * Remove stale lock files from the ~/.tilda/locks/ directory.
  *
- * @param home_directory the user's home directory
- *
  * Success: returns 0
  * Failure: returns non-zero
  */
-static gint remove_stale_lock_files (gchar *home_directory)
+static gint remove_stale_lock_files ()
 {
     DEBUG_FUNCTION ("remove_stale_lock_files");
-    DEBUG_ASSERT (home_directory != NULL);
 
     GSList *pids = NULL;
     FILE *ps_output;
@@ -223,7 +220,7 @@ static gint remove_stale_lock_files (gchar *home_directory)
     pclose (ps_output);
 
     struct lock_info *lock;
-    gchar *lock_dir = g_build_filename (home_directory, ".tilda", "locks", NULL);
+    gchar *lock_dir = g_build_filename (g_get_user_cache_dir (), "tilda", "locks", NULL);
     gchar *remove_file;
     gchar *filename;
     GDir *dir;
@@ -274,7 +271,7 @@ static gint remove_stale_lock_files (gchar *home_directory)
 static gboolean parse_cli (int argc, char *argv[])
 {
     DEBUG_FUNCTION ("parse_cli");
-    DEBUG_ASSERT (argc != NULL);
+    DEBUG_ASSERT (argc != 0);
     DEBUG_ASSERT (argv != NULL);
 
     /* Set default values */
@@ -297,20 +294,20 @@ static gboolean parse_cli (int argc, char *argv[])
 
     /* All of the various command-line options */
     GOptionEntry cl_opts[] = {
-        { "antialias",          'a', 0, G_OPTION_ARG_NONE,      &antialias,         _("Use Antialiased Fonts"), NULL },
-        { "background-color",   'b', 0, G_OPTION_ARG_STRING,    &background_color,  _("Set the background color"), NULL },
-        { "command",            'c', 0, G_OPTION_ARG_STRING,    &command,           _("Run a command at startup"), NULL },
-        { "hidden",             'h', 0, G_OPTION_ARG_NONE,      &hidden,            _("Start Tilda hidden"), NULL },
-        { "font",               'f', 0, G_OPTION_ARG_STRING,    &font,              _("Set the font to the following string"), NULL },
-        { "lines",              'l', 0, G_OPTION_ARG_INT,       &lines,             _("Scrollback Lines"), NULL },
-        { "scrollbar",          's', 0, G_OPTION_ARG_NONE,      &scrollbar,         _("Use Scrollbar"), NULL },
-        { "transparency",       't', 0, G_OPTION_ARG_INT,       &transparency,      _("Opaqueness: 0-100%"), NULL },
-        { "version",            'v', 0, G_OPTION_ARG_NONE,      &version,           _("Print the version, then exit"), NULL },
-        { "working-dir",        'w', 0, G_OPTION_ARG_STRING,    &working_dir,       _("Set Initial Working Directory"), NULL },
-        { "x-pos",              'x', 0, G_OPTION_ARG_INT,       &x_pos,             _("X Position"), NULL },
-        { "y-pos",              'y', 0, G_OPTION_ARG_INT,       &y_pos,             _("Y Position"), NULL },
-        { "image",              'B', 0, G_OPTION_ARG_STRING,    &image,             _("Set Background Image"), NULL },
-        { "config",             'C', 0, G_OPTION_ARG_NONE,      &show_config,       _("Show Configuration Wizard"), NULL },
+        { "antialias",          'a', 0, G_OPTION_ARG_NONE,      &antialias,         N_("Use Antialiased Fonts"), NULL },
+        { "background-color",   'b', 0, G_OPTION_ARG_STRING,    &background_color,  N_("Set the background color"), NULL },
+        { "command",            'c', 0, G_OPTION_ARG_STRING,    &command,           N_("Run a command at startup"), NULL },
+        { "hidden",             'h', 0, G_OPTION_ARG_NONE,      &hidden,            N_("Start Tilda hidden"), NULL },
+        { "font",               'f', 0, G_OPTION_ARG_STRING,    &font,              N_("Set the font to the following string"), NULL },
+        { "lines",              'l', 0, G_OPTION_ARG_INT,       &lines,             N_("Scrollback Lines"), NULL },
+        { "scrollbar",          's', 0, G_OPTION_ARG_NONE,      &scrollbar,         N_("Use Scrollbar"), NULL },
+        { "transparency",       't', 0, G_OPTION_ARG_INT,       &transparency,      N_("Opaqueness: 0-100%"), NULL },
+        { "version",            'v', 0, G_OPTION_ARG_NONE,      &version,           N_("Print the version, then exit"), NULL },
+        { "working-dir",        'w', 0, G_OPTION_ARG_STRING,    &working_dir,       N_("Set Initial Working Directory"), NULL },
+        { "x-pos",              'x', 0, G_OPTION_ARG_INT,       &x_pos,             N_("X Position"), NULL },
+        { "y-pos",              'y', 0, G_OPTION_ARG_INT,       &y_pos,             N_("Y Position"), NULL },
+        { "image",              'B', 0, G_OPTION_ARG_STRING,    &image,             N_("Set Background Image"), NULL },
+        { "config",             'C', 0, G_OPTION_ARG_NONE,      &show_config,       N_("Show Configuration Wizard"), NULL },
         { NULL }
     };
 
@@ -337,11 +334,12 @@ static gboolean parse_cli (int argc, char *argv[])
     {
         g_print ("%s\n\n", TILDA_VERSION);
 
-        g_print ("Copyright (c) 2005-2008 Tristan Sloughter (sloutri@iit.edu)\n");
-        g_print ("Copyright (c) 2005-2008 Ira W. Snyder (tilda@irasnyder.com)\n\n");
+        g_print ("Copyright (c) 2012-2013 Sebastian Geiger (lanoxx@gmx.net)\n");
+        g_print ("Copyright (c) 2005-2009 Tristan Sloughter (sloutri@iit.edu)\n");
+        g_print ("Copyright (c) 2005-2009 Ira W. Snyder (tilda@irasnyder.com)\n\n");
 
-        g_print ("General Information: http://launchpad.net/tilda\n");
-        g_print ("Bug Reports: http://bugs.launchpad.net/tilda\n\n");
+        g_print ("General Information: https://github.com/lanoxx/tilda\n");
+        g_print ("Bug Reports: https://github.com/lanoxx/tilda/issues?state=open\n\n");
 
         g_print ("This program comes with ABSOLUTELY NO WARRANTY.\n");
         g_print ("This is free software, and you are welcome to redistribute it\n");
@@ -388,27 +386,6 @@ static gboolean parse_cli (int argc, char *argv[])
     return show_config;
 }
 
-/*
- * Finds the coordinate that will center the tilda window in the screen.
- *
- * If you want to center the tilda window on the top or bottom of the screen,
- * pass the screen width into screen_dimension and the tilda window's width
- * into the tilda_dimension variable. The result will be the x coordinate that
- * should be used in order to have the tilda window centered on the screen.
- *
- * Centering based on y coordinate is similar, just use the screen height and
- * tilda window height.
- */
-int find_centering_coordinate (const int screen_dimension, const int tilda_dimension)
-{
-    DEBUG_FUNCTION ("find_centering_coordinate");
-
-    const float screen_center = screen_dimension / 2.0;
-    const float tilda_center  = tilda_dimension  / 2.0;
-
-    return screen_center - tilda_center;
-}
-
 /**
  * Get a pointer to the config file name for this instance.
  *
@@ -417,13 +394,20 @@ int find_centering_coordinate (const int screen_dimension, const int tilda_dimen
  * @param tw the tilda_window structure corresponding to this instance
  * @return a pointer to a string representation of the config file's name
  */
-static gchar *get_config_file_name (gchar *home_directory, gint instance)
+static gchar *get_config_file_name (gint instance)
 {
     DEBUG_FUNCTION ("get_config_file_name");
-    DEBUG_ASSERT (home_directory != NULL);
     DEBUG_ASSERT (instance >= 0);
 
-    gchar *config_file_prefix = g_build_filename (home_directory, ".tilda", "config_", NULL);
+    gchar *config_dir = g_build_filename (g_get_user_config_dir (), "tilda", NULL);
+
+    /* Make the ~/.config/tilda directory */
+    if (!g_file_test (config_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
+        g_print (_("Creating directory:'%s'\n"), config_dir);
+        g_mkdir_with_parents (config_dir,  S_IRUSR | S_IWUSR | S_IXUSR);
+    }
+
+    gchar *config_file_prefix = g_build_filename (config_dir, "config_", NULL);
     gchar *config_file;
 
     config_file = g_strdup_printf ("%s%d", config_file_prefix, instance);
@@ -441,10 +425,9 @@ static gchar *get_config_file_name (gchar *home_directory, gint instance)
  * Success: return next available instance number (>=0)
  * Failure: return 0
  */
-static gint get_instance_number (gchar *home_directory)
+static gint get_instance_number ()
 {
     DEBUG_FUNCTION ("get_instance_number");
-    DEBUG_ASSERT (home_directory != NULL);
 
     gint i;
     gchar *name;
@@ -452,7 +435,7 @@ static gint get_instance_number (gchar *home_directory)
     GDir *dir;
     GSList *list = NULL;
     struct lock_info *lock;
-    gchar *lock_dir = g_build_filename (home_directory, ".tilda", "locks", NULL);
+    gchar *lock_dir = g_build_filename (g_get_user_cache_dir (), "tilda", "locks", NULL);
 
     /* Open the lock directory */
     dir = g_dir_open (lock_dir, 0, NULL);
@@ -501,6 +484,49 @@ static void termination_handler (gint signum)
     gtk_main_quit ();
 }
 
+/*
+ * This is to do the migration of config files from ~/.tilda to the
+ * XDG_*_HOME folders
+ */
+static int migrate_config_files(char *old_config_path)
+{
+    gchar* old_lock_dir = g_build_filename(old_config_path, "locks", NULL);
+    gchar* new_lock_dir = g_build_filename(g_get_user_cache_dir (), "tilda", "locks", NULL);
+    gchar* new_config_dir = g_build_filename(g_get_user_config_dir (), "tilda", NULL);
+
+    if(!g_file_test(new_lock_dir, G_FILE_TEST_IS_DIR)) {
+           g_mkdir_with_parents(new_lock_dir, 0700);
+    }
+
+    //we basically need to move the files from old_config_path to config and cache
+    g_rename(old_lock_dir, new_lock_dir);
+    //we must move the config files after we have moved the locks directory, otherwise it gets moved aswell
+    g_rename(old_config_path, new_config_dir);
+    g_free(old_lock_dir);
+    g_free(new_lock_dir);
+    g_free(new_config_dir);
+}
+
+static void load_custom_css_file () {
+    GtkCssProvider *provider;
+    char* cssfilename = g_build_filename(
+        g_get_user_config_dir (), "tilda", "style.css", NULL);
+    if (g_file_test (cssfilename, G_FILE_TEST_EXISTS)) {
+        g_print (_("Found style.css in the user config directory, "
+            "applying user css stlye.\n"));
+        provider = gtk_css_provider_new ();
+        gtk_style_context_add_provider_for_screen (
+            gdk_screen_get_default(),
+            GTK_STYLE_PROVIDER(provider),
+            GTK_STYLE_PROVIDER_PRIORITY_USER
+        );
+        GFile *cssfile = g_file_new_for_path (cssfilename);
+        gtk_css_provider_load_from_file (GTK_CSS_PROVIDER (provider),
+            cssfile, NULL);
+        g_object_unref (cssfile);
+    }
+}
+
 int main (int argc, char *argv[])
 {
     DEBUG_FUNCTION ("main");
@@ -510,22 +536,34 @@ int main (int argc, char *argv[])
     struct sigaction sa;
     struct lock_info lock;
     gboolean need_wizard = FALSE;
-    gchar *home_dir, *config_file, *lock_file;
+    gchar *config_file, *lock_file, *old_config_path;
 
-    home_dir = g_strdup (g_get_home_dir ());
+    /*
+     * Migration code to move old files to new XDG folders
+     */
+    old_config_path = g_build_filename(g_get_home_dir (), ".tilda", NULL);
+    if (g_file_test(old_config_path, G_FILE_TEST_IS_DIR)) {
+        g_print (_("Migrating old config path to XDG folders\n"));
+        migrate_config_files(old_config_path);
+        g_free(old_config_path);
+    }
 
     /* Remove stale lock files */
-    remove_stale_lock_files (home_dir);
+    remove_stale_lock_files ();
 
     lock.pid = getpid ();
-    lock.instance = get_instance_number (home_dir);
-    config_file = get_config_file_name (home_dir, lock.instance);
-    lock_file = create_lock_file (home_dir, lock);
+    lock.instance = get_instance_number ();
+    config_file = get_config_file_name (lock.instance);
+    lock_file = create_lock_file (lock);
 
 #if ENABLE_NLS
     /* Gettext Initialization */
-    setlocale (LC_ALL, "");
+    char* locale = setlocale (LC_ALL, "");
+    if(!locale) {
+        g_warning ("Could not setup locale.");
+    }
     bindtextdomain (PACKAGE, LOCALEDIR);
+    bind_textdomain_codeset (PACKAGE, "UTF-8");
     textdomain (PACKAGE);
 #endif
 
@@ -545,17 +583,10 @@ int main (int argc, char *argv[])
     /* We're about to startup X, so set the error handler. */
     XSetErrorHandler (xerror_handler);
 
-    if (!g_thread_supported ())
-        g_thread_init(NULL);
-
     /* Initialize GTK and libglade */
     gtk_init (&argc, &argv);
 
-    /*
-     * Not needed with the new glade:
-     * ====================================
-     * glade_init ();
-     */
+    load_custom_css_file ();
 
     /* create new tilda_window */
     tw = tilda_window_init (config_file, lock.instance);
@@ -564,6 +595,9 @@ int main (int argc, char *argv[])
     if (tw == NULL)
         goto tw_alloc_failed;
 
+    /* Adding widget title for CSS selection */
+    gtk_widget_set_name (GTK_WIDGET(tw->window), "Main");
+
     /* Initialize and set up the keybinding to toggle tilda's visibility. */
     tomboy_keybinder_init ();
 
@@ -592,33 +626,24 @@ int main (int argc, char *argv[])
     /* Show the wizard if we need to.
      *
      * Note that the key will be bound upon exiting the wizard */
-    if (need_wizard)
+    if (need_wizard) {
+        g_print ("Starting the wizard to configure tilda options.");
         wizard (tw);
-    else
-    {
+    } else {
         gint ret = tilda_keygrabber_bind (config_getstr ("key"), tw);
 
         if (!ret)
         {
             /* The key was unbindable, so we need to show the wizard */
-            show_invalid_keybinding_dialog (NULL);
+            show_invalid_keybinding_dialog (NULL, _("The keybinding you chose for \"Pull Down Terminal\" is invalid. Please choose another."));
             wizard (tw);
         }
     }
 
-    if (config_getbool ("hidden"))
-    {
-        /* It does not cause graphical glitches to make tilda hidden on start this way.
-         * It does make tilda appear much faster on it's first appearance, so I'm leaving
-         * it this way, because it has a good benefit, and no apparent drawbacks. */
-        gtk_widget_show (GTK_WIDGET(tw->window));
-        gtk_widget_hide (GTK_WIDGET(tw->window));
-    }
-    else
-    {
-        pull (tw, PULL_DOWN);
-    }
+    pull (tw, config_getbool ("hidden") ? PULL_UP : PULL_DOWN);
 
+    g_print ("Tilda has started. Press %s to pull down the window.\n",
+        config_getstr ("key"));
     /* Whew! We're finally all set up and ready to run GTK ... */
     gtk_main();
 
@@ -630,7 +655,6 @@ tw_alloc_failed:
     g_remove (lock_file);
     g_free (lock_file);
     g_free (config_file);
-    g_free (home_dir);
 
     return 0;
 }
