@@ -17,34 +17,24 @@
 #ifndef TILDA_H
 #define TILDA_H
 
-#include <gtk/gtk.h>
-#include <confuse.h>
-#include "tilda_window.h"
+#include <tilda_window.h>
 
-G_BEGIN_DECLS;
+#include <gtk/gtk.h>
+
+G_BEGIN_DECLS
 
 #define TILDA_VERSION PACKAGE_NAME " " PACKAGE_VERSION
 #define QUICK_STRCMP(a, b) (*(a)!=*(b)? \
     (int) ((unsigned char) *(a) - (unsigned char) *(b)) : \
     strcmp ((a), (b)))
 
-gboolean image_set_clo, antialias_set_clo, scroll_set_clo;
-gint old_max_height, old_max_width;
+struct lock_info
+{
+    gint pid;
+    gint instance;
+};
 
-void clean_up (tilda_window *tw);
-void clean_up_no_args ();
 
-#define HEIGHT 0
-#define WIDTH  1
-
-#define CONFIG1_OLDER -1
-#define CONFIGS_SAME   0
-#define CONFIG1_NEWER  1
-
-int get_display_dimension (int dimension);
-#define get_physical_height_pixels() get_display_dimension(HEIGHT)
-#define get_physical_width_pixels()  get_display_dimension(WIDTH)
-
-G_END_DECLS;
+G_END_DECLS
 
 #endif
