@@ -22,23 +22,23 @@
 #include <tilda_terminal.h>
 #include <callback_func.h>
 #include <configsys.h>
-#include <translation.h>
 #include <wizard.h> /* wizard */
+#include <gtk/gtk.h>
 
 #include <stdio.h>
 #include <stdlib.h> /* malloc */
 #include <gtk/gtk.h>
 #include <glib-object.h>
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <vte/vte.h>
 #include <string.h>
 
+#define HTTP_REGEXP "(ftp|http)s?://[-a-zA-Z0-9.?$%&/=_~#.,:;+]*"
 
-#define DINGUS1 "(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?"
-#define DINGUS2 "(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?/[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#\\%]*[^]'\\.}>\\) ,\\\"]"
+GdkColor current_palette[TERMINAL_PALETTE_SIZE];
 
-
-
-static gint start_shell (struct tilda_term_ *tt);
+static gint start_shell (struct tilda_term_ *tt, gboolean ignore_custom_command);
 static gint tilda_term_config_defaults (tilda_term *tt);
 static void child_exited_cb (GtkWidget *widget, gpointer data);
 static void window_title_changed_cb (GtkWidget *widget, gpointer data);
@@ -76,6 +76,7 @@ struct tilda_term_ *tilda_term_init (struct tilda_window_ *tw)
 
     int ret;
     struct tilda_term_ *term;
+    GError *error = NULL;
 
     term = g_malloc (sizeof (struct tilda_term_));
 
@@ -87,13 +88,14 @@ struct tilda_term_ *tilda_term_init (struct tilda_window_ *tw)
     term->tw = tw;
 
     /* Create a non-homogenous hbox, with 0px spacing between members */
-    term->hbox = gtk_hbox_new (FALSE, 0);
+    term->hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
     /* Create the terminal */
     term->vte_term = vte_terminal_new ();
 
     /* Create the scrollbar for the terminal */
-    term->scrollbar = gtk_vscrollbar_new (VTE_TERMINAL(term->vte_term)->adjustment);
+    term->scrollbar = gtk_scrollbar_new (GTK_ORIENTATION_VERTICAL,
+        gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (VTE_TERMINAL(term->vte_term))));
 
     /* Set properties of the terminal */
     tilda_term_config_defaults (term);
@@ -143,9 +145,9 @@ struct tilda_term_ *tilda_term_init (struct tilda_window_ *tw)
                       G_CALLBACK(decrease_font_size_cb), tw->window);
 
     /* Match URL's, etc */
-    ret = vte_terminal_match_add (VTE_TERMINAL(term->vte_term), DINGUS1);
-    vte_terminal_match_set_cursor_type (VTE_TERMINAL(term->vte_term), ret, GDK_HAND2);
-    ret = vte_terminal_match_add(VTE_TERMINAL (term->vte_term), DINGUS2);
+
+    term->http_regexp=g_regex_new(HTTP_REGEXP, G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY, &error);
+    ret = vte_terminal_match_add_gregex(VTE_TERMINAL(term->vte_term), term->http_regexp,0);
     vte_terminal_match_set_cursor_type (VTE_TERMINAL(term->vte_term), ret, GDK_HAND2);
 
     /* Show the child widgets */
@@ -153,7 +155,7 @@ struct tilda_term_ *tilda_term_init (struct tilda_window_ *tw)
     gtk_widget_show (term->hbox);
 
     /* Fork the appropriate command into the terminal */
-    ret = start_shell (term);
+    ret = start_shell (term, FALSE);
 
     if (ret)
         goto err_fork;
@@ -220,8 +222,8 @@ static void iconify_window_cb (GtkWidget *widget, gpointer data)
     DEBUG_ASSERT (data != NULL);
 
     if (GTK_IS_WIDGET(data))
-        if ((GTK_WIDGET(data))->window)
-            gdk_window_iconify ((GTK_WIDGET(data))->window);
+        if (gtk_widget_get_window ((GTK_WIDGET (data))))
+            gdk_window_iconify (gtk_widget_get_window ((GTK_WIDGET (data))));
 }
 
 static void deiconify_window_cb (GtkWidget *widget, gpointer data)
@@ -230,8 +232,8 @@ static void deiconify_window_cb (GtkWidget *widget, gpointer data)
     DEBUG_ASSERT (data != NULL);
 
     if (GTK_IS_WIDGET(data))
-        if ((GTK_WIDGET(data))->window)
-            gdk_window_deiconify ((GTK_WIDGET(data))->window);
+        if (gtk_widget_get_window ((GTK_WIDGET (data))))
+            gdk_window_deiconify (gtk_widget_get_window ((GTK_WIDGET (data))));
 }
 
 static void raise_window_cb (GtkWidget *widget, gpointer data)
@@ -240,8 +242,8 @@ static void raise_window_cb (GtkWidget *widget, gpointer data)
     DEBUG_ASSERT (data != NULL);
 
     if (GTK_IS_WIDGET(data))
-        if ((GTK_WIDGET(data))->window)
-            gdk_window_raise ((GTK_WIDGET(data))->window);
+        if (gtk_widget_get_window (GTK_WIDGET (data)))
+            gdk_window_raise (gtk_widget_get_window (GTK_WIDGET (data)));
 }
 
 static void lower_window_cb (GtkWidget *widget, gpointer data)
@@ -250,8 +252,8 @@ static void lower_window_cb (GtkWidget *widget, gpointer data)
     DEBUG_ASSERT (data != NULL);
 
     if (GTK_IS_WIDGET(data))
-        if ((GTK_WIDGET(data))->window)
-            gdk_window_lower ((GTK_WIDGET(data))->window);
+        if (gtk_widget_get_window (GTK_WIDGET (data)))
+            gdk_window_lower (gtk_widget_get_window (GTK_WIDGET (data)));
 }
 
 static void maximize_window_cb (GtkWidget *widget, gpointer data)
@@ -260,8 +262,8 @@ static void maximize_window_cb (GtkWidget *widget, gpointer data)
     DEBUG_ASSERT (data != NULL);
 
     if (GTK_IS_WIDGET(data))
-        if ((GTK_WIDGET(data))->window)
-            gdk_window_maximize ((GTK_WIDGET(data))->window);
+        if (gtk_widget_get_window (GTK_WIDGET(data)))
+            gdk_window_maximize (gtk_widget_get_window (GTK_WIDGET(data)));
 }
 
 static void restore_window_cb (GtkWidget *widget, gpointer data)
@@ -270,8 +272,8 @@ static void restore_window_cb (GtkWidget *widget, gpointer data)
     DEBUG_ASSERT (data != NULL);
 
     if (GTK_IS_WIDGET(data))
-        if ((GTK_WIDGET(data))->window)
-            gdk_window_unmaximize ((GTK_WIDGET(data))->window);
+        if (gtk_widget_get_window (GTK_WIDGET (data)))
+            gdk_window_unmaximize (gtk_widget_get_window (GTK_WIDGET (data)));
 }
 
 static void refresh_window_cb (GtkWidget *widget, gpointer data)
@@ -282,12 +284,14 @@ static void refresh_window_cb (GtkWidget *widget, gpointer data)
     GdkRectangle rect;
     if (GTK_IS_WIDGET(data))
     {
-        if ((GTK_WIDGET(data))->window)
+        if (gtk_widget_get_window (GTK_WIDGET (data)))
         {
             rect.x = rect.y = 0;
-            rect.width = (GTK_WIDGET(data))->allocation.width;
-            rect.height = (GTK_WIDGET(data))->allocation.height;
-            gdk_window_invalidate_rect ((GTK_WIDGET(data))->window, &rect, TRUE);
+            GtkAllocation allocation;
+            gtk_widget_get_allocation (GTK_WIDGET (data), &allocation);
+            rect.width = allocation.width;
+            rect.height = allocation.height;
+            gdk_window_invalidate_rect (gtk_widget_get_window (GTK_WIDGET(data)), &rect, TRUE);
         }
     }
 }
@@ -298,8 +302,8 @@ static void move_window_cb (GtkWidget *widget, guint x, guint y, gpointer data)
     DEBUG_ASSERT (data != NULL);
 
     if (GTK_IS_WIDGET(data))
-        if ((GTK_WIDGET(data))->window)
-            gdk_window_move ((GTK_WIDGET(data))->window, x, y);
+        if (gtk_widget_get_window (GTK_WIDGET (data)))
+            gdk_window_move (gtk_widget_get_window (GTK_WIDGET (data)), x, y);
 }
 
 static void adjust_font_size (GtkWidget *widget, gpointer data, gint howmuch)
@@ -315,13 +319,13 @@ static void adjust_font_size (GtkWidget *widget, gpointer data, gint howmuch)
 
     /* Read the screen dimensions in cells. */
     terminal = VTE_TERMINAL(widget);
-    columns = terminal->column_count;
-    rows = terminal->row_count;
+    columns = vte_terminal_get_column_count (terminal);
+    rows = vte_terminal_get_row_count (terminal);
 
     /* Take into account padding and border overhead. */
     gtk_window_get_size(GTK_WINDOW(data), &owidth, &oheight);
-    owidth -= terminal->char_width * terminal->column_count;
-    oheight -= terminal->char_height * terminal->row_count;
+    owidth -= vte_terminal_get_char_width (terminal) * columns;
+    oheight -= vte_terminal_get_char_height (terminal) * rows;
 
     /* Calculate the new font size. */
     desired = pango_font_description_copy (vte_terminal_get_font(terminal));
@@ -364,7 +368,7 @@ static void decrease_font_size_cb (GtkWidget *widget, gpointer data)
  * SUCCESS: return 0
  * FAILURE: return non-zero
  */
-static gint start_shell (struct tilda_term_ *tt)
+static gint start_shell (struct tilda_term_ *tt, gboolean ignore_custom_command)
 {
     DEBUG_FUNCTION ("start_shell");
     DEBUG_ASSERT (tt != NULL);
@@ -376,7 +380,7 @@ static gint start_shell (struct tilda_term_ *tt)
 
     gchar *default_command;
 
-    if (config_getbool ("run_command"))
+    if (config_getbool ("run_command") && !ignore_custom_command)
     {
         ret = g_shell_parse_argv (config_getstr ("command"), &argc, &argv, &error);
 
@@ -390,12 +394,24 @@ static gint start_shell (struct tilda_term_ *tt)
             goto launch_default_shell;
         }
 
-        ret = vte_terminal_fork_command (VTE_TERMINAL(tt->vte_term),
-                    argv[0], argv, NULL,
-                    config_getstr ("working_dir"),
-                    TRUE, TRUE, TRUE);
+        char **envv = malloc(2*sizeof(void *));
+        envv[0] = getenv("PATH");
+        envv[1] = NULL;
+
+        ret = vte_terminal_fork_command_full (VTE_TERMINAL (tt->vte_term),
+            VTE_PTY_DEFAULT, /* VtePtyFlags pty_flags */
+            config_getstr ("working_dir"), /* const char *working_directory */
+            argv, /* char **argv */
+            envv, /* char **envv */
+            G_SPAWN_SEARCH_PATH,    /* GSpawnFlags spawn_flags */
+            NULL, /* GSpawnChildSetupFunc child_setup */
+            NULL, /* gpointer child_setup_data */
+            NULL, /* GPid *child_pid */
+            NULL  /* GError **error */
+            );
 
         g_strfreev (argv);
+        g_free (envv);
 
         /* Check for error */
         if (ret == -1)
@@ -418,10 +434,26 @@ launch_default_shell:
     if (default_command == NULL)
         default_command = "/bin/sh";
 
-    ret = vte_terminal_fork_command (VTE_TERMINAL(tt->vte_term),
-                default_command, NULL, NULL,
-                config_getstr ("working_dir"),
-                TRUE, TRUE, TRUE);
+    /* We need to create a NULL terminated list of arguments.
+     * The first item is the command to execute in the shell, in this
+     * case there are no further arguments being passed. */
+    argv = malloc(2 * sizeof(void *));
+    argv[0] = default_command;
+    argv[1] = NULL;
+
+    ret = vte_terminal_fork_command_full (VTE_TERMINAL (tt->vte_term),
+        VTE_PTY_DEFAULT, /* VtePtyFlags pty_flags */
+        config_getstr ("working_dir"), /* const char *working_directory */
+        argv, /* char **argv */
+        NULL, /* char **envv */
+        0,    /* GSpawnFlags spawn_flags */
+        NULL, /* GSpawnChildSetupFunc child_setup */
+        NULL, /* gpointer child_setup_data */
+        NULL, /* GPid *child_pid */
+        NULL  /* GError **error */
+        );
+
+    g_free (argv);
 
     if (ret == -1)
     {
@@ -451,21 +483,21 @@ static void child_exited_cb (GtkWidget *widget, gpointer data)
 
     /* These can stay here. They don't need to go into a header because
      * they are only used at this point in the code. */
-    enum command_exit { HOLD_TERMINAL_OPEN, RESTART_COMMAND, EXIT_TERMINAL };
+    enum command_exit { DROP_TO_DEFAULT_SHELL, RESTART_COMMAND, EXIT_TERMINAL };
 
     /* Check the user's preference for what to do when the child terminal
      * is closed. Take the appropriate action */
     switch (config_getint ("command_exit"))
     {
         case EXIT_TERMINAL:
-            tilda_window_close_tab (tt->tw, index);
+            tilda_window_close_tab (tt->tw, index, FALSE);
             break;
         case RESTART_COMMAND:
             vte_terminal_feed (VTE_TERMINAL(tt->vte_term), "\r\n\r\n", 4);
-            start_shell (tt);
-            break;
-        case HOLD_TERMINAL_OPEN:
+            start_shell (tt, FALSE);
             break;
+        case DROP_TO_DEFAULT_SHELL:
+            start_shell (tt, TRUE);
         default:
             break;
     }
@@ -486,9 +518,10 @@ static gint tilda_term_config_defaults (tilda_term *tt)
 
     gdouble transparency_level = 0.0;
     GdkColor fg, bg /*, tint, highlight, cursor, black */;
+    gchar* word_chars;
+    gint i;
 
-
-    /** Colors **/
+    /** Colors & Palette **/
     bg.red   =    config_getint ("back_red");
     bg.green =    config_getint ("back_green");
     bg.blue  =    config_getint ("back_blue");
@@ -497,14 +530,23 @@ static gint tilda_term_config_defaults (tilda_term *tt)
     fg.green =    config_getint ("text_green");
     fg.blue  =    config_getint ("text_blue");
 
-    vte_terminal_set_colors (VTE_TERMINAL(tt->vte_term), &fg, &bg, NULL, 0);
+    for(i = 0;i < TERMINAL_PALETTE_SIZE; i++)
+    {
+        current_palette[i].pixel = 0;
+        current_palette[i].red   = config_getnint ("palette", i*3);
+        current_palette[i].green = config_getnint ("palette", i*3+1);
+        current_palette[i].blue  = config_getnint ("palette", i*3+2);
+    }
+
+    vte_terminal_set_colors (VTE_TERMINAL(tt->vte_term), &fg, &bg, current_palette, TERMINAL_PALETTE_SIZE);
 
     /** Bells **/
     vte_terminal_set_audible_bell (VTE_TERMINAL(tt->vte_term), config_getbool ("bell"));
     vte_terminal_set_visible_bell (VTE_TERMINAL(tt->vte_term), config_getbool ("bell"));
 
     /** Cursor **/
-    vte_terminal_set_cursor_blinks (VTE_TERMINAL(tt->vte_term), config_getbool ("blinks"));
+    vte_terminal_set_cursor_blink_mode (VTE_TERMINAL(tt->vte_term),
+            (config_getbool ("blinks"))?VTE_CURSOR_BLINK_ON:VTE_CURSOR_BLINK_OFF);
 
     /** Scrolling **/
     vte_terminal_set_scroll_background (VTE_TERMINAL(tt->vte_term), config_getbool ("scroll_background"));
@@ -517,7 +559,9 @@ static gint tilda_term_config_defaults (tilda_term *tt)
     /** Text Properties **/
     vte_terminal_set_allow_bold (VTE_TERMINAL(tt->vte_term), config_getbool ("bold"));
     gtk_widget_set_double_buffered (tt->vte_term, config_getbool("double_buffer"));
-    vte_terminal_set_font_from_string_full (VTE_TERMINAL(tt->vte_term), config_getstr ("font"), config_getbool ("antialias"));
+    PangoFontDescription *description =
+        pango_font_description_from_string (config_getstr ("font"));
+    vte_terminal_set_font (VTE_TERMINAL (tt->vte_term), description);
 
     /** Scrollback **/
     vte_terminal_set_scrollback_lines (VTE_TERMINAL(tt->vte_term), config_getint ("lines"));
@@ -557,6 +601,12 @@ static gint tilda_term_config_defaults (tilda_term *tt)
             break;
     }
 
+    /** Word chars **/
+    word_chars =  config_getstr ("word_chars");
+    if (NULL == word_chars || '\0' == word_chars)
+        word_chars = DEFAULT_WORD_CHARS;
+    vte_terminal_set_word_chars (VTE_TERMINAL(tt->vte_term), word_chars);
+
     /** Background **/
     if (config_getbool ("use_image"))
         vte_terminal_set_background_image_file (VTE_TERMINAL(tt->vte_term), config_getstr ("image"));
@@ -629,6 +679,16 @@ menu_add_tab_cb (GtkWidget *widget, gpointer data)
     tilda_window_add_tab (TILDA_WINDOW(data));
 }
 
+static void
+menu_fullscreen_cb (GtkWidget *widget, gpointer data)
+{
+    DEBUG_FUNCTION ("menu_fullscreen_cb");
+    DEBUG_ASSERT (widget != NULL);
+    DEBUG_ASSERT (data != NULL);
+
+    toggle_fullscreen_cb (TILDA_WINDOW(data));
+}
+
 static void
 menu_close_tab_cb (GtkWidget *widget, gpointer data)
 {
@@ -639,6 +699,16 @@ menu_close_tab_cb (GtkWidget *widget, gpointer data)
     tilda_window_close_current_tab (TILDA_WINDOW(data));
 }
 
+static void on_popup_hide (GtkWidget *widget, gpointer data)
+{
+    DEBUG_FUNCTION("on_popup_hide");
+    DEBUG_ASSERT(widget != NULL);
+    DEBUG_ASSERT(data != NULL);
+
+    tilda_window *tw = TILDA_WINDOW(data);
+    tw->disable_auto_hide = FALSE;
+}
+
 static void popup_menu (tilda_window *tw, tilda_term *tt)
 {
     DEBUG_FUNCTION ("popup_menu");
@@ -662,6 +732,8 @@ static void popup_menu (tilda_window *tw, tilda_term *tt)
                 "<menuitem action=\"copy\" />"
                 "<menuitem action=\"paste\" />"
                 "<separator />"
+                "<menuitem action=\"fullscreen\" />"
+                "<separator />"
                 "<menuitem action=\"preferences\" />"
                 "<separator />"
                 "<menuitem action=\"quit\" />"
@@ -673,27 +745,31 @@ static void popup_menu (tilda_window *tw, tilda_term *tt)
 
     /* Add Actions and connect callbacks */
     action = gtk_action_new ("new-tab", _("_New Tab"), NULL, GTK_STOCK_ADD);
-    gtk_action_group_add_action_with_accel (action_group, action, "<Ctrl><Shift>t");
+    gtk_action_group_add_action_with_accel (action_group, action, config_getstr("addtab_key"));
     g_signal_connect (G_OBJECT(action), "activate", G_CALLBACK(menu_add_tab_cb), tw);
 
     action = gtk_action_new ("close-tab", _("_Close Tab"), NULL, GTK_STOCK_CLOSE);
-    gtk_action_group_add_action_with_accel (action_group, action, "<Ctrl><Shift>w");
+    gtk_action_group_add_action_with_accel (action_group, action, config_getstr("closetab_key"));
     g_signal_connect (G_OBJECT(action), "activate", G_CALLBACK(menu_close_tab_cb), tw);
 
     action = gtk_action_new ("copy", NULL, NULL, GTK_STOCK_COPY);
-    gtk_action_group_add_action_with_accel (action_group, action, "<Ctrl><Shift>c");
+    gtk_action_group_add_action_with_accel (action_group, action, config_getstr("copy_key"));
     g_signal_connect (G_OBJECT(action), "activate", G_CALLBACK(menu_copy_cb), tt);
 
     action = gtk_action_new ("paste", NULL, NULL, GTK_STOCK_PASTE);
-    gtk_action_group_add_action_with_accel (action_group, action, "<Ctrl><Shift>v");
+    gtk_action_group_add_action_with_accel (action_group, action, config_getstr("paste_key"));
     g_signal_connect (G_OBJECT(action), "activate", G_CALLBACK(menu_paste_cb), tt);
 
+    action = gtk_action_new ("fullscreen", _("Toggle fullscreen"), NULL, NULL);
+    gtk_action_group_add_action (action_group, action);
+    g_signal_connect (G_OBJECT(action), "activate", G_CALLBACK(menu_fullscreen_cb), tw);
+
     action = gtk_action_new ("preferences", NULL, NULL, GTK_STOCK_PREFERENCES);
     gtk_action_group_add_action (action_group, action);
     g_signal_connect (G_OBJECT(action), "activate", G_CALLBACK(menu_preferences_cb), tw);
 
     action = gtk_action_new ("quit", NULL, NULL, GTK_STOCK_QUIT);
-    gtk_action_group_add_action_with_accel (action_group, action, "<Ctrl><Shift>q");
+    gtk_action_group_add_action_with_accel (action_group, action, config_getstr("quit_key"));
     g_signal_connect (G_OBJECT(action), "activate", G_CALLBACK(menu_quit_cb), tw);
 
     /* Create and add actions to the GtkUIManager */
@@ -712,6 +788,10 @@ static void popup_menu (tilda_window *tw, tilda_term *tt)
     /* Get the popup menu out of the GtkUIManager */
     menu = gtk_ui_manager_get_widget (ui_manager, "/ui/popup-menu");
 
+    /* Disable auto hide */
+    tw->disable_auto_hide = TRUE;
+    g_signal_connect (G_OBJECT(menu), "unmap", G_CALLBACK(on_popup_hide), tw);
+
     /* Display the menu */
     gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time());
     gtk_widget_show_all(menu);
@@ -742,16 +822,20 @@ static int button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer d
             break;
         case 1: /* Left Click */
             terminal  = VTE_TERMINAL(tt->vte_term);
-            vte_terminal_get_padding (terminal, &xpad, &ypad);
+            GtkBorder border;
+            gtk_widget_style_get (GTK_WIDGET (terminal),
+                "inner-border", &border, NULL);
+            xpad = border.left;
+            ypad = border.bottom;
             match = vte_terminal_match_check (terminal,
                     (event->x - ypad) /
-                    terminal->char_width,
+                    vte_terminal_get_char_width (terminal),
                     (event->y - ypad) /
-                    terminal->char_height,
+                    vte_terminal_get_char_height (terminal),
                     &tag);
 
             /* Check if we can launch a web browser, and do so if possible */
-            if ((event->state & GDK_CONTROL_MASK) && match != NULL)
+            if (match != NULL)
             {
 #if DEBUG
                 g_print ("Got a Ctrl+Left Click -- Matched: `%s' (%d)\n", match, tag);
